"""empty message

Revision ID: 925771029620
Revises: fb6b6f188497
Create Date: 2018-09-26 21:16:04.776712

"""

from alembic import op


# revision identifiers, used by Alembic.
revision = "925771029620"
down_revision = "fb6b6f188497"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("playlist_uc", "playlist", type_="unique")
    op.create_unique_constraint(
        "playlist_uc", "playlist", ["name", "project_id", "episode_id"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("playlist_uc", "playlist", type_="unique")
    op.create_unique_constraint(
        "playlist_uc", "playlist", ["name", "project_id"]
    )
    # ### end Alembic commands ###
