"""empty message

Revision ID: 443d1e78a932
Revises: 59e149a772cf
Create Date: 2018-08-07 21:36:03.384838

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "443d1e78a932"
down_revision = "c726b98be194"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "task_type", sa.Column("allow_timelog", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_type", "allow_timelog")
    # ### end Alembic commands ###
