"""empty message

Revision ID: 6f6049877105
Revises: 5b0fcbb94f24
Create Date: 2019-03-15 12:24:15.277023

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "6f6049877105"
down_revision = "5b0fcbb94f24"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("notification_uc", "notification", type_="unique")
    op.create_unique_constraint(
        "notification_uc",
        "notification",
        ["person_id", "author_id", "comment_id", "type"],
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("notification_uc", "notification", type_="unique")
    op.create_unique_constraint(
        "notification_uc",
        "notification",
        ["person_id", "author_id", "comment_id"],
    )
    # ### end Alembic commands ###
